/*
  roulette prototype for AVR32DD20
  
  LED layout 
       4  5
     3      6
   2          7
  18          8
    17      10
      16  11

  switch input 19
  
  created 2011.9.19 by ikkei
*/


const int buttonPin = 19;

const int led_num = 12;
byte led_pin[led_num] = {2, 3, 4, 5, 6, 7, 9, 10, 11, 16, 17, 18};

const int dsp_num = 12;

uint16_t led_out[dsp_num] = {
  0b000000000001,
  0b000000000010,
  0b000000000100,
  0b000000001000,
  0b000000010000,
  0b000000100000,
  0b000001000000,
  0b000010000000,
  0b000100000000,
  0b001000000000,
  0b010000000000,
  0b100000000000,
};
uint16_t led_data;
int dsp_count;

const int state_max = 14;
int interval[state_max] = { 9, 14, 21, 31, 47, 71, 107, 161, 241, 362, 544, 818, 500, 80};
int state_no = 0;

const int blink = 5;
int blink_count;
const int led_off = 0x00;

int i_count = 1;

void setup(){
  TCB0.CCMP   = 36000; // 1/24MHz * 36000 = 1.5ms
  TCB0.CTRLB = 0;
  TCB0.CTRLA = TCB_CLKSEL_DIV1_gc | TCB_ENABLE_bm; // 24MHz

  for(byte i=0; i<led_num; i++){
    pinMode(led_pin[i], OUTPUT);
  }
  pinMode(buttonPin, INPUT_PULLUP);
}

// LED output
void dsp_out(int data){
  for (byte i=0; i<led_num; i++){
    digitalWrite( led_pin[i], bitRead(data, i) ^ 0x01);
  }
}

// buttonPin input
byte polling_input(){
  static byte data0 = 0;
  static byte data1 = 0;
  static byte data2 = 0;
  static byte data3 = 0;
  static byte data4 = 0;
 
  data4 = data3;
  data2 = data1;
  data1 = data0;
  data0 = ~digitalRead(buttonPin); // negative input
  // eliminate chattering  チャタリング除去（３回読み）
  if ( data0 == data1 && data0 == data2 ){
    data3 = data0;
  }
  // positive edge detection   ポジティブエッジ検出
  return (~data4 & data3);
}

void loop(){
  // timer wait  タイマー待ち
  if (TCB0.INTFLAGS & TCB_CAPT_bm){
    TCB0.INTFLAGS = TCB_CAPT_bm;
    // LED output
    dsp_out( led_data );
    // input
    if (polling_input() == 1){
      if (state_no == 0){
        state_no = 1;
      } else {
        state_no = 0;
        i_count = 1;
      }
    }
    //
    i_count--;
    if (i_count == 0){
      i_count = interval[ state_no ];
      if (state_no < state_max -1 ){
        if (state_no > 0){
          state_no++;
        }
        dsp_count++;
        if (dsp_count >= dsp_num){
          dsp_count = 0;
        }
        led_data = led_out[dsp_count];
        blink_count = blink;
      } else {
        if (blink_count > 0){
          blink_count--;
          if ((blink_count % 2) == 0){
            led_data = led_out[dsp_count];
          } else {
            led_data = led_off;
          }
        }
      }
    }
  }
}

  
